// Copyright (c) 2022 Conviva All rights reserved.
//
// This program is licensed to you under the Apache License Version 2.0,
// and you may not use this file except in compliance with the Apache License Version 2.0.
// You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the Apache License Version 2.0 is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.

import Foundation
import ConvivaAppAnalytics

struct TrackerConfigurationReader: Decodable {
//    let appName: String?
    let devicePlatform: String?
    let base64Encoding: Bool?
    let platformContext: Bool?
    let geoLocationContext: Bool?
    let sessionContext: Bool?
    let userAnonymisation: Bool?
    let screenContext: Bool?
    let applicationContext: Bool?

    var devicePlatformType: CATDevicePlatform? {
        if let devicePlatform = self.devicePlatform { return CATStringToDevicePlatform(devicePlatform) }
        return nil
    }
}

extension TrackerConfigurationReader {
    static func defaultConfiguration(appName: String?) -> CATTrackerConfiguration {
        let trackerConfig = CATTrackerConfiguration()
        trackerConfig.trackerVersionSuffix(TrackerVersion.TRACKER_VERSION)

        trackerConfig.screenViewAutotracking(false)
        trackerConfig.lifecycleAutotracking(false)
        trackerConfig.installAutotracking(false)
        trackerConfig.exceptionAutotracking(false)
        trackerConfig.diagnosticAutotracking(false)
//        trackerConfig.userAnonymisation(false)
        
        if let _ = appName {
            trackerConfig.appId = appName!;
        }

        return trackerConfig
    }

    func toConfiguration(appName: String?) -> CATTrackerConfiguration {
        let trackerConfig = TrackerConfigurationReader.defaultConfiguration(appName: appName)

        if let dp = self.devicePlatformType { trackerConfig.devicePlatform(dp) }
        if let enc = self.base64Encoding { trackerConfig.base64Encoding(enc) }
        if let pc = self.platformContext { trackerConfig.platformContext(pc) }
        if let gc = self.geoLocationContext { trackerConfig.geoLocationContext(gc) }
        if let sc = self.sessionContext { trackerConfig.sessionContext(sc) }
//        if let ua = self.userAnonymisation { trackerConfig.userAnonymisation(ua) }
        if let scr = self.screenContext { trackerConfig.screenContext(scr) }
        if let ac = self.applicationContext { trackerConfig.applicationContext(ac) }

        return trackerConfig
    }
}
