// Copyright (c) 2022 Conviva All rights reserved.
//
// This program is licensed to you under the Apache License Version 2.0,
// and you may not use this file except in compliance with the Apache License Version 2.0.
// You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the Apache License Version 2.0 is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.

import 'dart:js_util';
import 'dart:html';

import 'readers/configurations/configuration_reader.dart';
import 'readers/messages/event_message_reader.dart';
import 'readers/messages/set_user_id_message_reader.dart';
import 'readers/messages/track_custom_event_reader.dart';
import 'convivaAppTracker.dart';

class ConvivaTrackerController {
  /*static void createTracker(ConfigurationReader configuration) {
    dynamic options = configuration.getTrackerOptions();
    conviva('newTracker', configuration.namespace, configuration.networkConfig.endpoint, jsify(options));

    if (configuration.subjectConfig != null &&
        configuration.subjectConfig?.userId != null) {
      _setUserId(configuration.namespace, configuration.subjectConfig?.userId);
    }

    if (configuration.gdprConfig != null) {
      conviva(
          'enableGdprContext',
          jsify({
            'basisForProcessing': configuration.gdprConfig?.basisForProcessing,
            'documentId': configuration.gdprConfig?.documentId,
            'documentVersion': configuration.gdprConfig?.documentVersion,
            'documentDescription': configuration.gdprConfig?.documentDescription
          }));
    }

    if (configuration
            .trackerConfig?.webActivityTracking?.enableActivityTracking ??
        false) {
      final webActivityTracking =
          configuration.trackerConfig!.webActivityTracking!;
      conviva(
          'enableActivityTracking',
          jsify({
            'minimumVisitLength': webActivityTracking.minimumVisitLength,
            'heartbeatDelay': webActivityTracking.heartbeatDelay
          }));
    }
  }*/

  static void createConvivaTracker(ConfigurationReader configuration) {
    dynamic options = configuration.getTrackerOptions();
    options['appId'] = configuration.appName;
    options['convivaCustomerKey'] = configuration.customerKey;
    conviva('convivaAppTracker', jsify(options));

    if (configuration.subjectConfig != null &&
        configuration.subjectConfig?.userId != null) {
      _setUserId("CAT", configuration.subjectConfig?.userId);
    }
    if (configuration.networkConfig != null) {
      // Add network config if present and pass to tracker core.
    }

    if (configuration.gdprConfig != null) {
      conviva(
          'enableGdprContext',
          jsify({
            'basisForProcessing': configuration.gdprConfig?.basisForProcessing,
            'documentId': configuration.gdprConfig?.documentId,
            'documentVersion': configuration.gdprConfig?.documentVersion,
            'documentDescription': configuration.gdprConfig?.documentDescription
          }));
    }

    if (configuration
            .trackerConfig?.webActivityTracking?.enableActivityTracking ??
        false) {
      final webActivityTracking =
          configuration.trackerConfig!.webActivityTracking!;
      conviva(
          'enableActivityTracking',
          jsify({
            'minimumVisitLength': webActivityTracking.minimumVisitLength,
            'heartbeatDelay': webActivityTracking.heartbeatDelay
          }));
    }
  }

  static void trackEvent(EventMessageReader message) {
    conviva('${message.event().endpoint()}',
        jsify(message.eventData()));
  }

  static void setUserId(SetUserIdMessageReader message) {
    _setUserId(message.tracker, message.userId);
  }

  static void trackCustomEvent(TrackCustomEventMessageReader message) {
    conviva('trackCustomEvent', jsify({
      'name': message.name,
      'data': message.data
    }));
  }

  static void _setUserId(String tracker, String? userId) {
    conviva('setUserId', userId);
  }

  static String? getSessionUserId() {
    return _getConvivaCookieParts()?[0];
  }

  static String? getSessionId() {
    return _getConvivaCookieParts()?[5];
  }

  static int? getSessionIndex() {
    final cookiePart = _getConvivaCookieParts()?[2];
    if (cookiePart != null) {
      return int.tryParse(cookiePart);
    }
    return null;
  }

  static List<String>? _getConvivaCookieParts() {
    final regex = RegExp(r'_sp_id\.[a-f0-9]+=([^;]+);?');
    if (document.cookie != null) {
      final cookieValue = regex.firstMatch(document.cookie!)?.group(1);
      return cookieValue?.split('.');
    }
    return null;
  }
}
